package Crypt::ECB;


########################################
# general module startup things
########################################

use strict;
use vars qw(@ISA @EXPORT_OK $VERSION);

require Exporter;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(encrypt decrypt encrypt_hex decrypt_hex);
$VERSION   = 1.00;


########################################
# basic methods
########################################

#
# constructor, initialization of vars
#
sub new ($)
{
    my $class = shift;
    my $crypt = {
	Mode      => '',
	Key       => '',
	Method    => '',
	Blocksize => '',
	Errstring => '',
    };

    bless $crypt, $class;
    return $crypt;
}

#
# sets mode if argument given, either en- or decrypt
# returns mode
#
sub mode ($;$)
{
    my $crypt = shift;

    if (my $mode = shift)
    {
	unless ($mode=~/^[de]/i)
	{
	    $crypt->{Errstring} = 
		"Mode has to be either (e)ncrypt or (d)ecrypt.\n";
	    return 0;
	}

	$mode = ($mode=~/^d/i) ? "decrypt" : "encrypt";
            # checks whether $mode starts with either d or e,
            # only first character matters, rest of $mode is ignored
	$crypt->{Mode} = $mode;
    }

    return $crypt->{Mode};
}

#
# sets key if argument given
# returns key
#
sub key ($;$)
{
    my $crypt = shift;
    $crypt->{Key} = shift if @_;
    return $crypt->{Key};
}

#
# sets and loads crypting module if argument given
# returns module name
#
sub method ($;$)
{
    my $crypt = shift;

    if (my $method = shift)
    {
	my $module = "Crypt::$method";
	eval "require $module";
	if ($@)
	{
	    $crypt->{Errstring} = "Couldn't load $module: $@"
		. "Install $module in the proper path or"
		. " choose some other crypting method.\n";
	    return 0;
	}

	unless ($module->can('blocksize'))
	{
	    $crypt->{Errstring} =
		"Can't work because $module doesn't report blocksize."
		    . " Are you sure $module is a valid crypting"
		    . " module?\n";
	    return 0;
	}

	$crypt->{Blocksize} = $module->blocksize;
	$crypt->{Method}    = $method;
    }

    return $crypt->{Method};
}

#
# calls the crypting module
# returns the en-/decrypted date
#
sub process ($;$)
{
    my $crypt  = shift;
    my $data = shift || $_;

    my $bs     = $crypt->{Blocksize};
    my $key    = $crypt->{Key};
    my $mode   = $crypt->{Mode};
    my $method = $crypt->{Method};
    my $module = "Crypt::$method";

    unless ($key) {
	$crypt->{Errstring} = "Key not set. Use '\$your_obj->key"
	    . "('some_key'). Possibly the key length is specified"
	    . " by the algorithm (for example 'Blowfish' needs an"
	    . " eight byte key).\n";
	return 0;
    }

    unless ($mode) {
	$crypt->{Errstring} = "Mode not set."
	    . " Use '\$your_obj->mode(\$mode)', \$mode being either"
	    . " 'encrypt' or 'decrypt'.\n";
	return 0;
    }
    unless ($method) {
	$crypt->{Errstring} = "Method not set."
	    . " Use '\$your_obj->method(\$method)', \$method being"
	    . " some algorithm like for example 'DES', 'IDEA' or"
	    . " 'Blowfish'. The corresponding module 'Crypt::"
	    . "\$method' has to be installed.\n";
	return 0;
    }

    my @blocks = $data=~/(.{1,$bs})/ogs;
        # data is split into blocks of proper size
        # blocksize is provided by the cipher module

    $blocks[$#blocks] = pack("a$bs",$blocks[$#blocks]);
        # last block is filled with binary 0 up to blocksize

    my $cipher = $module->new($key);
    my $text;
    foreach my $block (@blocks) {
	$text .= $cipher->$mode($block);
    }
    return $text;
}

#
# returns errstring
#
sub errstring ($)
{
    my $crypt = shift;
    return $crypt->{Errstring};
}


########################################
# convenience functions/methods
########################################

#
# dies with usage msg
#
sub error
{
    die "Usage: en-/decrypt('key','method','data').\n";
}

#
# magic convenience encrypt function/method
#
sub encrypt ($;$$)
{
    if (ref ($_[0]))
    {
	my $crypt = shift;
	$crypt->mode('encrypt');

	if (my $data = shift) { $crypt->process($data) };
    }
    else
    {
	error unless @_==3;
	my ($key, $method, $data) = @_;

	my $crypt = Crypt::ECB->new;
	$crypt->key($key);
	$crypt->method($method) || die $crypt->errstring; 
	$crypt->mode('encrypt');
	return $crypt->process($data);
    }
}

#
# magic convenience decrypt function/method
#
sub decrypt ($;$$)
{
    if (ref ($_[0]))
    {
	my $crypt = shift;
	$crypt->mode('decrypt');

	if (my $data = shift) { $crypt->process($data) };
    }
    else
    {
	error unless @_==3;
	my ($key, $method, $data) = @_;

	my $crypt = Crypt::ECB->new;
	$crypt->key($key);
	$crypt->method($method) || die $crypt->errstring;
	$crypt->mode('decrypt');
	return $crypt->process($data);
    }
}

#
# calls encrypt, returns hex packed data
#
sub encrypt_hex ($$;$)
{
    if (ref ($_[0]))
    {
	my $crypt = shift;
	join('',unpack('H*',$crypt->encrypt(@_)));
    }
    else
    {
	error unless @_==3;
	join('',unpack('H*',encrypt($_[0], $_[1], $_[2])));
    }
}

#
# calls decrypt, expected input is hex packed
#
sub decrypt_hex ($$;$)
{
    if (ref ($_[0]))
    {
	my $crypt = shift;
	$crypt->decrypt(pack('H*',));
    }
    else
    {
	error unless @_==3;
	decrypt($_[0], $_[1], pack('H*',$_[2]));
    }
}


########################################
# finally, to satisfy require
########################################

1;
__END__


=head1 NAME

Crypt::ECB - Encrypt Data using ECB Mode

=head1 SYNOPSIS

  use Crypt::ECB;

  $crypt = Crypt::ECB->new;
  $crypt->method('Blowfish');
  $crypt->key('some_key'); 

  $enc = $crypt->encrypt("Some data.");
  print  $crypt->decrypt($enc);

or

  use Crypt::ECB qw(encrypt decrypt encrypt_hex decrypt_hex);

  $ciphertext = encrypt $key2, "Blowfish", "Some data";
  $plaintext  = decrypt $key2, "Blowfish", $ciphertext;

  $hexcode = encrypt_hex "1x2Y3z", "IDEA", $plaintext;
  $plain   = decrypt_hex "1x2Y3z", "IDEA", "A01B45BC";

=head1 DESCRIPTION

This module is a Perl-only implementation of the ECB mode.  In
combination with a block cipher such as DES, IDEA or Blowfish, you can
encrypt and decrypt messages of arbitrarily long length.  The encrypted
messages are compatible with the encryption format used by B<SSLeay>.
Though for security reasons other modes than ECB such as CBC should be
preferred.  See textbooks on cryptography if you want to know why.

The functionality of the module can be accessed via OO methods or via
standard function calls.  Remember that some crypting module like for
example Blowfish has to be installed.

=head1 METHODS

=head2 new()

  $crypt = Crypt::ECB->new;

B<new()> is called without parameters.  The method just initializes
the variables it uses.

=head2 key()

  $crypt->key('some_key');

  print $crypt->key;

This method sets the key if given a parameter.  It always returns the
key.  Note that some crypting modules require keys of definite length.
For example the Blowfish algorithm expects an eight byte key. 

=head2 method()

  $crypt->method('Blowfish);

  print $crypt->method;

This method sets the crypting algorithm to be used if given a parameter.
It the tries to load the corresponding module.  If an error occurs, it
returns 0 and sets $crypt->{Errstring}.  Otherwise it returns the
algorithm name.  Free packages available for Perl are for example
Blowfish or TripleDES

=head2 mode()

  $crypt->mode('encrypt');
  $crypt->mode('decrypt');

  print $crypt->mode;

This method sets the crypting mode if given an argument.  Allowed are
any words starting either with 'e' or 'd'.  The Method returns the mode
which is set.  Alternatively B<encrypt()> or B<decrypt()> can be used.

=head2 process()

  $crypt->mode('encrypt');
  $enc = $crypt->process($data);

  $crypt->mode('decrypt');
  print $crypt->process($enc);

B<process()> processes the data given as argument.  If called without
argument $_ is processed.  The method returns the processed data.
Alternatively B<encrypt()> or B<decrypt()> can be used.  Module, mode
and key have to be set in order to be able to process data.  If some
of these are not set, the method returns 0 and sets $crypt->{Errstring}.

=head2 encrypt(), decrypt()

  $crypt->encrypt;
  $enc = $crypt->process($data);

  $crypt->decrypt;
  print $crypt->process($enc);

is equivalent to

  $enc = $crypt->encrypt($data);
  print $crypt->decrypt($enc);

If called without argument, these methods are equivalent to
$crypt->mode('en-/decrypt').  If given an argument, the argument is
processed after having set the mode.

=head2 encrypt_hex(), decrypt_hex()

  $hexenc = $crypt->encrypt_hex($data);
  print $crypt->decrypt_hex($hexenc);

These are convenience functions that operate on ciphertext in a
hexadecimal representation.  These functions can be useful if, for
example, you wish to place the encrypted information into an e-mail
message, Web page or URL.

=head2 errstring()

  print $crypt->errstring;

If in another method an error occurs, the method sets
$crypt->{Errstring} which can be read by calling B<errstring()>.

=head1 VARIABLES

Variables which are used by the module are:

  $crypt->{Key},
  $crypt->{Method},
  $crypt->{Blocksize},
  $crypt->{Mode},
  $crypt->{Errstring}.

These variables could be set and read directly, but you should do
that only if you really know what you are doing (well, reading should
not pose any problems).  Use instead the methods described above.

=head1 FUNCTIONS

For convenience en- or decrypting can also be done by calling ordinary
functions.  The functions are: B<encrypt()>, B<decrypt()>,
B<encrypt_hex>, B<decrypt_hex>.  The module is smart enough to
recognize whether these functions are called in an OO context or not.

=head2 encrypt(), decrypt()

  $ciphertext = encrypt $key, $method, $plaintext;
  $plaintext  = decrypt $key, $method, $ciphertext;

These functions process the provided text and return either the
corresponding ciphertext (encrypt) or plaintext (decrypt).

=head2 encrypt_hex(), decrypt_hex()

  $ciphertext = encrypt_hex $key, $method, $plaintext;
  $plaintext  = decrypt_hex $key, $method, $ciphertext;

These functions operate on ciphertext in a hexadecimal representation.

=head1 BUGS

None that I know of.

=head1 AUTHOR

Christoph Appel, cappel@debis.com

=head1 SEE ALSO

perl(1), Crypt::DES(3), Crypt::IDEA(3), Crypt::CBC(3)

=cut
